
/*===============================================*\
	Samedi 7 novembre 1998

	Arash HABIBI

	Medium.c

	Ouvre deux sockets, et ecrit dans l'une 
	tout ce qu'il peut lire sur l'autre et 
	reciproquement. Pourra eventuellement 
	y ajouter des erreurs. 
	On suppose que les sockets distantes 
	s'appellent sock_emit et sock_rec et 
	qu'elles sont dans le meme repertoire. 
\*================================================*/


#include "SocketToolbox.h"
#include "CodageDecodage.h"
#include "Erreur.h"
#include <unistd.h>
#include <stdio.h>
#include <time.h>


void WaitForAnotherPoint(int s_rec, int s_em, struct sockaddr_un adr_em)
/*===============================================*\
  Attend encore une seconde pour un second point. 
  Si oui, alors il l'envoie au recepteur, sinon rien.
\*===============================================*/
{
  int taille, taille_adresse=110, nonbloquant=1; 
  struct sockaddr_un adr_rec; 
  char message_code[MAX_MSG_CODE]; 

  sleep(1);

  /* Pour pouvoir lire sur la socket, sans rester bloque */
  ioctl(s_rec, FIONBIO, &nonbloquant);

  taille = recvfrom(s_rec, message_code, MAX_MSG_CODE, 0, 
		    (struct sockaddr *) &adr_rec,
		    &taille_adresse); 

  if((taille==1)&&(message_code[0]=='.'))
    {
      if(sendto(s_em, ".", 1, 0, 
		   (struct sockaddr*) &adr_em, 
		   sizeof(adr_em))<0)
	{perror("sendto \'.\' 2"); exit(1);}
    }
}
  
  

int Echange(int s_rec, int s_em, struct sockaddr_un adr_em, int TypeDErreur)
/*===============================================*\
  D'apres select nous savons qu'il y a 
  quelque chose a lire sur s_rec. 
  Le but de cette fonction est de lire
  ce quelque chose et de l'envoyer sur 
  s_em. 
  La fonction rend -1 en cas d'erreur,
  1 si tout va bien, et 0 quand une des
  sockets envoie une trame qui ne contient 
  qu'un seul caractere '.' 
\*===============================================*/
{
    int nb_rec, nb_emis, taille_adresse=110; 
    char message_code[MAX_MSG_CODE]; 
    struct sockaddr_un adr_rec; 
    
    nb_rec = recvfrom(s_rec, message_code, MAX_MSG_CODE, 0, 
		    (struct sockaddr *) &adr_rec,
		    &taille_adresse); 
    
    if(nb_rec<0) {perror("read socket"); 
    	return(-1);}

    /*----- Traitement de la fin de la transmission -----*/

    if((nb_rec==1)&&(message_code[0]=='.'))
      {
	if(sendto(s_em, ".", 1, 0, 
		  (struct sockaddr*) &adr_em, 
		  sizeof(adr_em))<0)
	  {perror("sendto \'.\' 1"); return(-1);}
	
	WaitForAnotherPoint(s_rec, s_em, adr_em);
	
	return(0);
      }

    /*----- Traitement de la fin de la transmission -----*/
    
    else if(nb_rec>=0)
	{
	    IntroduitErreur(message_code,nb_rec, TypeDErreur);

	    nb_emis = sendto(s_em, message_code, nb_rec, 0, 
			     (struct sockaddr*) &adr_em, 
			     sizeof(adr_em));

	    if(nb_emis<0) {perror("sendto"); return(-1);}
	    
	    else return(1);
	}
    return 0;
}



int main(int argc, char **argv)
/*===============================================*\
  Ouvre et prepare deux sockets unix, 
\*===============================================*/
{
    fd_set lecture, ecriture, except;
    struct sockaddr_un adr_dist_emit, adr_dist_rec; 
    int s_medemit, s_medrec, nbfds, nb_events,
      stop=0, crash=0, res, TypeDErreur; 

    if((argc!=1)&&(argc!=2))
	{printf("Usage : medium\n");	
	 printf("ou      medium <typederreur> \n");	
	 exit(1);}

    if(argc==1) TypeDErreur = 0; /* pas d'erreur */
    else TypeDErreur = atoi(argv[1]); 
    

    /**
       initialisation du generateur aleatoire !!
    */
    srand48(time(0));
/* ----- creation et attachement des sockets --------------*/
/* ----- preparation de l'adresse de la socket distante -- */

    s_medemit = GetAndBindSocket("s_medemit");
    s_medrec  = GetAndBindSocket("s_medrec");

    adr_dist_emit = PrepareAdresse("sock_emit"); 
    adr_dist_rec  = PrepareAdresse("sock_rec"); 
    
/* ----- Preparation de select ----------------------------*/

    if(s_medemit>s_medrec) nbfds = s_medemit + 1; 
    else                   nbfds = s_medrec  + 1; 

/* ----- Echanges de donnees ------------------------------*/

    printf("Medium pret\n");	    
    
    while((!stop)&&(!crash))
	{
	    /*---- Preparation des ensembles -------- */
	    /*--------- de descripteurs ------------- */
	    
	    FD_ZERO(&lecture); 
	    FD_ZERO(&ecriture); 
	    FD_ZERO(&except); 
	    FD_SET(s_medemit, &lecture); 
	    FD_SET(s_medrec , &lecture); 
	    
	    /*------- attente d'un evenment --------- */

	    nb_events = select (nbfds, &lecture, &ecriture, 
				&except, NULL);
	    if(nb_events==-1) {perror("select"); crash=1;} 
	    
	    /*------- le depouillement des votes ---- */
	    
	    if(FD_ISSET(s_medemit, &lecture))
		{
		  res = Echange(s_medemit, s_medrec, 
				adr_dist_rec, TypeDErreur); 
		  if(res==0) stop=1; 
		  else if(res<0) crash=1; 
		}
	    
	    if(FD_ISSET(s_medrec, &lecture))
		{	
		  res = Echange(s_medrec, s_medemit, 
				adr_dist_emit, TypeDErreur); 
		  if(res==0) stop=1; 
		  else if(res<0) crash=1; 
		}
	}

    if(crash)
	printf("Echange interrompu\n\n");
    else if(stop)
	printf("Echange termine. Deconnexion.\n\n");
    
    close(s_medrec);
    close(s_medemit);
    remove("s_medemit");     
    remove("s_medrec"); 
    return 0;
}
